package eu.jonahbauer.json.tokenizer.reader;

import org.jetbrains.annotations.Range;

import java.io.IOException;

public interface PushbackReader extends AutoCloseable {
    int EOF = -1;

    /**
     * Reads the next character.
     * @return the next character or {@link #EOF} if the end of the stream has been reached
     * @throws IOException if an I/O error occurs
     */
    @Range(from = -1, to = Character.MAX_VALUE) int read() throws IOException;

    /**
     * Pushes the reader back, making {@link #read()} return the same character as before.
     * @throws IllegalStateException when called before the first read or more than once after each read
     */
    void pushback();

    /**
     * {@return the line number of the current character}
     * @throws IllegalStateException when called before the first read
     */
    @Range(from = 1, to = Integer.MAX_VALUE) int getLineNumber();

    /**
     * {@return the column number of the current character}
     * @throws IllegalStateException when called before the first read
     */
    @Range(from = 1, to = Integer.MAX_VALUE) int getColumnNumber();

    @Override
    void close() throws IOException;
}
